; =================================================================================================
; Title:   ExcelHost.asm
; Author:  G. Friedrich
; Version: 1.0.0
; Purpose: Demonstration of ExcelHost using ObjAsm32.
; =================================================================================================


%include @Environ(OA32_PATH)\\Code\\Macros\\Model.inc       ;Include & initialize standard modules
SysSetup OOP_PRIMER;, DEBUG(WND)                             ;Loads OOP files and OS related objects

IncludeBoth advapi32, ole32, oleaut32                       ;Includes these APIs

% include &MacPath&UStrings.inc                             ;UNICODE string support
% include &MacPath&BStrings.inc                             ;BSTR support
% include &COMPath&COM.inc                                  ;COM basic support
% include &COMPath&COM_Interfaces.inc                       ;COM common interfaces
% include &COMPath&OAIDL.inc
% include &COMPath&COM_Dispatch.inc                         ;Dispatch definitions and macros
% include &COMPath&COM_Dispatch.asm                         ;Dispatch procedures

.const                                                      ;Global GUIDs
DefGUID IID_NULL, %sGUID_NULL
DefGUID IID_IUnknown, %sIID_IUnknown

.data
vString     VARIANT   {}
pBook       Pointer   NULL
pSheet      Pointer   NULL
pRange      Pointer   NULL
dBStrSize   dword     0
bBuffer     byte 20 dup (?)
pChart      Pointer   NULL
vArray1     VARIANT   {}
vArray2     VARIANT   {}

MakeObjects ExcelHost

.code
Start:
    ;***** ObjAsm32 initialization ***** 
    SysInit

    ;***** OLE / COM initialization *****
    invoke CoInitialize, 0

    ;***** ExcelHost initialization *****
    OCall @ExcelHost::ExcelHost.Init, NULL
    OCall @ExcelHost::ExcelHost.SetVisible, TRUE

    ;***** Create a new WorkBook and a new WorkSheet *****
    mov pBook, $OCall(@ExcelHost::ExcelHost.NewBook)
    mov pSheet, $OCall(@ExcelHost::ExcelHost.NewSheet, pBook, $OfsCBStr("My Worksheet"))
;    OCall @ExcelHost::ExcelHost.GetSheetName, pSheet
;    DbgUStr eax
;    invoke SysFreeString, eax

    ;***** Get a Range and format it *****
    mov pRange, $OCall(@ExcelHost::ExcelHost.GetRange, pSheet, $OfsCBStr("C3:E5"))
;    OCall @ExcelHost::ExcelHost.SelectRange, pRange
    OCall @ExcelHost::ExcelHost.SetCell, pRange, 2, 2, $OfsCBStr("Hello Excel")
    OCall @ExcelHost::ExcelHost.SetRangeAlignHor, pRange, xlHAlignCenter
    OCall @ExcelHost::ExcelHost.SetRangeAlignVer, pRange, xlHAlignCenter
    
    ;***** Make a Border arround the Range *****
    mov ebx, $OCall(@ExcelHost::ExcelHost.GetRangeBorder, pRange, xlEdgeTop)
    OCall @ExcelHost::ExcelHost.SetBorderAttr, ebx, xlDouble, xlThick, $RGB(255,0,0)
    ICall ebx::Border.Release
    mov ebx, $OCall(@ExcelHost::ExcelHost.GetRangeBorder, pRange, xlEdgeRight)
    OCall @ExcelHost::ExcelHost.SetBorderAttr, ebx, xlDouble, xlThick, $RGB(255,0,0)
    ICall ebx::Border.Release
    mov ebx, $OCall(@ExcelHost::ExcelHost.GetRangeBorder, pRange, xlEdgeBottom)
    OCall @ExcelHost::ExcelHost.SetBorderAttr, ebx, xlDouble, xlThick, $RGB(255,0,0)
    ICall ebx::Border.Release
    mov ebx, $OCall(@ExcelHost::ExcelHost.GetRangeBorder, pRange, xlEdgeLeft)
    OCall @ExcelHost::ExcelHost.SetBorderAttr, ebx, xlDouble, xlThick, $RGB(255,0,0)
    ICall ebx::Border.Release
    
    ;***** Set the Range Interior *****
    mov ebx, $OCall(@ExcelHost::ExcelHost.GetInterior, pRange)
    OCall @ExcelHost::ExcelHost.SetInteriorAttr, ebx, $RGB(250,128,128), xlPatternLightDown, $RGB(0,0,255)
    ICall ebx::Interior.Release
    
    ;***** Set the Range Font *****
    mov ebx, $OCall(@ExcelHost::ExcelHost.GetFont, pRange)
    OCall @ExcelHost::ExcelHost.SetFontAttr, ebx, $OfsCBStr("Arial"), XlFontStyleItalic or XlFontStyleBold, 20, $RGB(255,0,0), xlUnderlineStyleSingle
    ICall ebx::Font.Release

;    OCall @ExcelHost::ExcelHost.GetCell, pRange, 2, 2, addr vString, VT_BSTR
;    DbgUStr vString.VARIANT.bstrVal
;    invoke SysFreeString, vString.VARIANT.bstrVal

    ICall pRange::IUnknown.Release
    
    ;***** Format Rows and Columns *****
    OCall @ExcelHost::ExcelHost.SetColumnWidth, pSheet, $OfsCBStr("C:E"), $CFloat4(20.0)
    OCall @ExcelHost::ExcelHost.SetRowHeight, pSheet, $OfsCBStr("3:5"), $CFloat4(40.0)

    ;******************************
    ;***** Create a new Chart *****
    ;******************************
    mov pChart, $OCall(@ExcelHost::ExcelHost.NewChart, xlXYScatterLinesNoMarkers, xlLocationAsObject, $OfsCBStr("My Worksheet"))
    
    ;***** Fill the Chart data range *****
    mov pRange, $OCall(@ExcelHost::ExcelHost.GetRange, pSheet, $OfsCBStr("A7:B36"))
    mov ebx, 1
    .while ebx <= 30
      invoke dword2dec, addr bBuffer, ebx
      invoke Str2BStr, addr bBuffer, addr bBuffer
      OCall @ExcelHost::ExcelHost.SetCell, pRange, ebx, 1, addr bBuffer
      lea eax, [2*ebx]
      invoke dword2dec, addr bBuffer, eax
      invoke Str2BStr, addr bBuffer, addr bBuffer
      OCall @ExcelHost::ExcelHost.SetCell, pRange, ebx, 2, addr bBuffer
      inc ebx
    .endw
    
    OCall @ExcelHost::ExcelHost.SetChartData, pChart, pRange, xlColumns
    
    ;***** Set Chart Title ***** 
    OCall @ExcelHost::ExcelHost.HasChartTitle, pChart, TRUE
    mov edi, $OCall(@ExcelHost::ExcelHost.GetChartTitle, pChart)
    OCall @ExcelHost::ExcelHost.SetChartTitleText, edi, $OfsCBStr("My Chart")

    ;***** Format Chart Title ***** 
    mov ebx, $OCall(@ExcelHost::ExcelHost.GetFont, edi)
    OCall @ExcelHost::ExcelHost.SetFontAttr, ebx, $OfsCBStr("Arial"), 0, 14, $RGB(127,127,127), xlUnderlineStyleSingle
    ICall ebx::Font.Release
    mov ebx, $OCall(@ExcelHost::ExcelHost.GetInterior, edi)
    OCall @ExcelHost::ExcelHost.SetInteriorAttr, ebx, $RGB(210,210,210), xlPatternSolid, $RGB(0,0,0)
    ICall ebx::Interior.Release
    mov ebx, $OCall(@ExcelHost::ExcelHost.GetBorder, edi)
    OCall @ExcelHost::ExcelHost.SetBorderAttr, ebx, xlNone, xlThin, $RGB(127,127,127)
    ICall ebx::Border.Release
    ICall edi::ChartTitle.Release
    
    ;***** Format Chart Area ***** 
    mov edi, $OCall(@ExcelHost::ExcelHost.GetChartArea, pChart)
    mov ebx, $OCall(@ExcelHost::ExcelHost.GetInterior, edi)
    OCall @ExcelHost::ExcelHost.SetInteriorAttr, ebx, $RGB(200,200,200), xlPatternSolid, $RGB(0,0,0)
    ICall ebx::Interior.Release
    mov ebx, $OCall(@ExcelHost::ExcelHost.GetBorder, edi)
    OCall @ExcelHost::ExcelHost.SetBorderAttr, ebx, xlContinuous, xlThin, $RGB(127,127,127)
    ICall ebx::Border.Release
    ICall edi::ChartArea.Release
    
    ;***** Format Chart Plot Area ***** 
    mov edi, $OCall(@ExcelHost::ExcelHost.GetChartPlotArea, pChart)
    mov ebx, $OCall(@ExcelHost::ExcelHost.GetInterior, edi)
    OCall @ExcelHost::ExcelHost.SetInteriorAttr, ebx, $RGB(255,255,255), xlPatternSolid, $RGB(0,0,0)
    ICall ebx::Interior.Release
    mov ebx, $OCall(@ExcelHost::ExcelHost.GetBorder, edi)
    OCall @ExcelHost::ExcelHost.SetBorderAttr, ebx, xlContinuous, xlNormal, $RGB(127,127,127)
    ICall ebx::Border.Release
    ICall edi::ChartArea.Release
    
    ;***** Format Chart X-Axis *****
    mov ebx, $OCall(@ExcelHost::ExcelHost.GetChartAxis, pChart, xlCategory, xlPrimary)
    mov edi, $OCall(@ExcelHost::ExcelHost.GetBorder, ebx)
    OCall @ExcelHost::ExcelHost.SetBorderAttr, edi, xlContinuous, xlThin, $RGB(0,0,255)
    OCall @ExcelHost::ExcelHost.SetChartAxisValues, ebx, FALSE, $CFloat8(0.0), FALSE, $CFloat8(10.0)
    OCall @ExcelHost::ExcelHost.SetChartAxisAttr, ebx, xlLinear, xlNone, FALSE, xlLow
    OCall @ExcelHost::ExcelHost.SetChartAxisTicks, ebx, xlOutside, FALSE, $CFloat8(2.0), xlOutside, FALSE, $CFloat8(5.0)
    OCall @ExcelHost::ExcelHost.SetChartAxisGridlines, ebx, xlMajorGridlines, TRUE, xlContinuous, xlThick, $RGB(0,255,0) 
    OCall @ExcelHost::ExcelHost.SetChartAxisGridlines, ebx, xlMinorGridlines, TRUE, xlDot, xlThin, $RGB(128,128,128) 
    
    ICall edi::Border.Release
    ICall ebx::Axis.Release

    ;***** Format Chart Y-Axis *****
    mov ebx, $OCall(@ExcelHost::ExcelHost.GetChartAxis, pChart, xlValue, xlPrimary)
    mov edi, $OCall(@ExcelHost::ExcelHost.GetBorder, ebx)
    OCall @ExcelHost::ExcelHost.SetBorderAttr, edi, xlContinuous, xlThin, $RGB(0,0,255)
    ICall edi::Border.Release
    OCall @ExcelHost::ExcelHost.SetChartAxisValues, ebx, FALSE, $CFloat8(0.0), FALSE, $CFloat8(15.0)
    OCall @ExcelHost::ExcelHost.SetChartAxisAttr, ebx, xlLinear, xlNone, FALSE, xlRight
    OCall @ExcelHost::ExcelHost.SetChartAxisTicks, ebx, xlOutside, FALSE, $CFloat8(1.0), xlOutside, FALSE, $CFloat8(5.0)
    OCall @ExcelHost::ExcelHost.SetChartAxisGridlines, ebx, xlMajorGridlines, TRUE, xlContinuous, xlThick, $RGB(0,255,0) 
    OCall @ExcelHost::ExcelHost.SetChartAxisGridlines, ebx, xlMinorGridlines, TRUE, xlDot, xlThin, $RGB(128,128,128) 
    
    ICall ebx::Axis.Release

    ;***** Format Chart Legend *****
    mov ebx, $OCall(@ExcelHost::ExcelHost.GetChartLegend, pChart)
    mov edi, $OCall(@ExcelHost::ExcelHost.GetBorder, ebx)
    ICall edi::Border.Release
    mov edi, $OCall(@ExcelHost::ExcelHost.GetInterior, ebx)
    OCall @ExcelHost::ExcelHost.SetInteriorAttr, edi, $RGB(255,255,255), xlPatternSolid, $RGB(0,0,0)
    ICall edi::Interior.Release
    mov edi, $OCall(@ExcelHost::ExcelHost.GetFont, ebx)
    OCall @ExcelHost::ExcelHost.SetFontAttr, edi, $OfsCBStr("Arial"), 0, 10, $RGB(128,128,128), xlUnderlineStyleNone
    ICall edi::Font.Release
    ICall ebx::Legend.Release
    
    ;***** Format Chart Series *****
    mov ebx, $OCall(@ExcelHost::ExcelHost.GetChartSeries, pChart, 1)
    OCall @ExcelHost::ExcelHost.SetChartSeriesAttr, ebx, xlDiamond, 7, $RGB(0,127,0), $RGB(0,255,0), TRUE, FALSE
    mov edi, $OCall(@ExcelHost::ExcelHost.GetBorder, ebx)
    OCall @ExcelHost::ExcelHost.SetBorderAttr, edi, xlContinuous, xlNormal, $RGB(0,127,0)
    ICall edi::Border.Release
    
    ;***** Set new Chart Series *****
    mov pRange, $OCall(@ExcelHost::ExcelHost.GetRange, pSheet, $OfsCBStr("A7:A36"))
    invoke VariantInit, addr vArray1
    OCall @ExcelHost::ExcelHost.GetRangeArray, pRange, addr vArray1
    ICall pRange::Range.Release
    mov pRange, $OCall(@ExcelHost::ExcelHost.GetRange, pSheet, $OfsCBStr("B7:B36"))
    invoke VariantInit, addr vArray2
    OCall @ExcelHost::ExcelHost.GetRangeArray, pRange, addr vArray2
    ICall pRange::Range.Release
    OCall @ExcelHost::ExcelHost.SetChartSeriesData, ebx, $OfsCBStr('="My Series"'), addr vArray1, addr vArray2
    invoke SafeArrayDestroy, vArray1.VARIANT.pDispVal
    invoke SafeArrayDestroy, vArray2.VARIANT.pDispVal
    ICall ebx::Series.Release

    ;***** Chart Cleanup *****
    ICall pChart::IUnknown.Release

    ;***** Set Saved flag ******
;    OCall @ExcelHost::ExcelHost.SaveAs, pBook, $OfsCBStr("TestBook.xls")
    OCall @ExcelHost::ExcelHost.SetSavedFlag, pBook, TRUE


    @invoke MessageBox, NULL, "Press OK to continue", "Pause", MB_OK
    ;***** Cleanup *****
    ICall pSheet::IUnknown.Release
    ICall pBook::IUnknown.Release

    ;***** Quit Excel application *****
    OCall @ExcelHost::ExcelHost.Quit

    ;***** Terminate ExcelHost object *****
    OCall @ExcelHost::ExcelHost.Done

    ;***** OLE / COM cleanup *****
    invoke CoUninitialize

    ;***** ObjAsm32 cleanup *****
    SysDone

    invoke ExitProcess, 0
end Start